/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BITINDICES_EXCEPTION_H
#define BITINDICES_EXCEPTION_H

#include <sstream>

#include <bit/except.h>

namespace bit
  {

  /**
   * Base class for all indices exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  struct indices_exception: public bit_exception
    {
      indices_exception(): bit_exception("bit:Indices: Unknown exception.")
      { }
      indices_exception(const std::string s): bit_exception(s)
      { }
    }
  ;

  namespace exception
    {


    namespace indices
      {

      /**
       * Invalid numeric indices identifier
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct invalid_numeric_id: public indices_exception
        {
          invalid_numeric_id(): indices_exception("bit:Indices: Invalid numeric identifier, index not found.")
          { }
          invalid_numeric_id(unsigned int id): indices_exception("bit:Indices: Invalid numeric identifier, index not found.")
          {
            std::ostringstream sout;
            sout << "bit:Indices: No index with numeric identifier (" << id << ")";
            m_exception_string = sout.str();
          }
        }
      ;

      /**
       * Invalid string indices identifier
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct invalid_string_id: public indices_exception
        {
          invalid_string_id(): indices_exception("bit:Indices: Invalid string identifier, index not found.")
          { }
          invalid_string_id(std::string s): indices_exception("bit:Indices: Invalid numeric identifier, index not found.")
          {
            m_exception_string = "bit:Indices: No index with string identifier (" + s + ")";
          }
        }
      ;


    }
  }
}
#endif
