/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BITTYPE_EXCEPTION_H
#define BITTYPE_EXCEPTION_H

#include <sstream>

#include <bit/except.h>

namespace bit
  {

  /**
   * Base struct for all type exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  struct type_exception: public bit_exception
    {
      type_exception(): bit_exception("bit:Type: Unknown type exception.")
      { }
      type_exception(const std::string s): bit_exception(s)
      { }
    }
  ;

  namespace exception
    {


    namespace type
      {

      /**
       * Integer length: Integer length must be <= 64
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct integer_length: public type_exception
        {
          integer_length(): type_exception("bit:Type: Integer length must be <= 64.")
          { }
        }
      ;

      /**
       * Bad floating point length: length must be 32 or 64
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct floating_point_length: public type_exception
      {
          floating_point_length(): type_exception("bit:Type: Bad floating point length. Length must be 32 or 64.")
          { }
      }
      ;

    }
  }
}
#endif
