/***************************************************************************
 *   Copyright (C) 2004,2010 by Rick L. Vinyard, Jr.                       *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include <bit/bit-config.h>

#ifndef BITPOINTER
#define BITPOINTER
  // Headers for smart pointers
  #ifdef BIT_HAVE_STDCXX_0X_SHARED_POINTER
    #include <memory>
    #define BitPointer std::shared_ptr
    #define BitWeakPointer std::weak_ptr
    #define bit_static_pointer_cast  std::static_pointer_cast
    #define bit_const_pointer_cast   std::const_pointer_cast
    #define bit_dynamic_pointer_cast std::dynamic_pointer_cast
  #else
    #ifdef BIT_HAVE_STDCXX_0X_TR1_SHARED_POINTER
      #include <typeinfo>
      #include <memory>
      #include <functional>
      #include <bits/concurrence.h>
      #include <ext/mt_allocator.h>
      #include <tr1/boost_shared_ptr.h>
      #define BitPointer std::tr1::shared_ptr
      #define BitWeakPointer std::tr1::weak_ptr
      #define bit_static_pointer_cast  std::tr1::static_pointer_cast
      #define bit_const_pointer_cast   std::tr1::const_pointer_cast
      #define bit_dynamic_pointer_cast std::tr1::dynamic_pointer_cast
    #else
      #ifdef BIT_HAVE_STDCXX_0X_BOOST_SHARED_POINTER
        #include <boost/shared_ptr.hpp>
        #include <boost/weak_ptr.hpp>
        #define BitPointer boost::shared_ptr
        #define BitWeakPointer boost::weak_ptr
        #define bit_static_pointer_cast  boost::static_pointer_cast
        #define bit_const_pointer_cast   boost::const_pointer_cast
        #define bit_dynamic_pointer_cast boost::dynamic_pointer_cast
      #endif
    #endif
  #endif
#endif
