/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BITRECORDSTORAGE_H
#define BITRECORDSTORAGE_H

#include <set>
#include <map>

#include <bit/fieldbase.h>
#include <bit/recordbase.h>
#include <bit/except.h>

namespace bit {

/**
 * Implements a storage mechanism that provides Field set management and also provides
 * efficient string name lookup of elements.
 *
 * @ingroup records
 * @author Rick L. Vinyard, Jr. <rvinyard@cs.nmsu.edu>
 */
class RecordStorage{
public:
    RecordStorage(FieldBase& record);

    virtual ~RecordStorage();

    struct fieldbase_pointer_compare {
      bool operator() (FieldBase::pointer p1, FieldBase::pointer p2) const {
        int p1_offset, p2_offset;
        p1_offset = p1->offset(BITS);
        p2_offset = p2->offset(BITS);
        if (p1_offset < p2_offset)
          return true;
        else if (p1_offset == p2_offset)
          return ( p1->length(BITS) < p2->length(BITS) );
        else
          return false;
      }
    };

    virtual FieldBase::iterator begin();

    virtual FieldBase::iterator end();

    /** Performs a deep copy and clone of fields */
    RecordStorage& operator=(const RecordStorage& other);

    virtual size_t subfields();

    virtual FieldBase::pointer subfield(const std::string& s);

    virtual FieldBase::pointer subfield(size_t i);

    virtual void add_field(FieldBase::pointer field);

    virtual void remove_field(FieldBase::pointer field);

    virtual void remove_field(const std::string& name);

    /** Typedef to container used to store children */
    typedef std::set<FieldBase::pointer, fieldbase_pointer_compare> Fields;

    /** Typedef to container used to store mapping from string names to a specific child */
    typedef std::map<std::string, Fields::iterator> FieldNameMap;

    Fields field_set;
    FieldNameMap field_name_map;

    virtual FieldBase::pointer previous_field(FieldBase::pointer current_field) throw (exception::invalid_container_op);

    virtual FieldBase::pointer next_field(FieldBase::pointer current_field) throw (exception::invalid_container_op);

  protected:

    FieldBase* m_record;

};

}

#endif
